<?php

namespace Drupal\pager\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\image\Entity\ImageStyle;
use Drupal\pager\PagerStorage;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Suggestion configuration form.
 */
class PagerAdminForm extends FormBase {
  protected $dbh;
  protected $factory;

  /**
   * Class constructor.
   *
   * @param \Drupal\pager\PagerStorage $dbh
   *   The storage interface.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $factory
   *   The pager settings.
   */
  public function __construct(PagerStorage $dbh, ConfigFactoryInterface $factory) {
    $this->dbh = $dbh;
    $this->factory = $factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static($container->get('pager.storage'), $container->get('config.factory'));
  }

  /**
   * The suggestion configuration form.
   *
   * @param array $form
   *   A drupal form array.
   * @param Drupal\Core\Form\FormStateInterface $form_state
   *   A Drupal form state object.
   *
   * @return array
   *   A Drupal form array.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $cfg = $this->getConfig();

    $form['field'] = [
      '#title'         => $this->t('Image Field'),
      '#type'          => 'select',
      '#options'       => $this->getImageFields(),
      '#default_value' => $cfg->field,
      '#required'      => TRUE,
      '#weight'        => 10,
    ];
    $form['style'] = [
      '#title'         => $this->t('Image Style'),
      '#type'          => 'select',
      '#options'       => $this->getImageStyles(),
      '#default_value' => $cfg->style,
      '#required'      => TRUE,
      '#weight'        => 20,
    ];
    $form['types'] = [
      '#title'         => $this->t('Content Types'),
      '#type'          => 'checkboxes',
      '#options'       => $this->getContentTypes(),
      '#default_value' => $cfg->types,
      '#required'      => TRUE,
      '#weight'        => 30,
    ];
    $form['terms'] = [
      '#title'         => $this->t('Taxonomy Terms'),
      '#type'          => 'checkboxes',
      '#options'       => $this->getTerms(),
      '#default_value' => $cfg->terms,
      '#required'      => TRUE,
      '#weight'        => 40,
    ];
    $form['submit'] = [
      '#type'   => 'submit',
      '#value'  => $this->t('Submit'),
      '#weight' => 100,
    ];
    return $form;
  }

  /**
   * The form ID.
   *
   * @return string
   *   The form ID.
   */
  public function getFormId() {
    return 'pager_admin';
  }

  /**
   * Submit function for the suggestion configuration form.
   *
   * @param array $form
   *   A drupal form array.
   * @param Drupal\Core\Form\FormStateInterface $form_state
   *   A Drupal form state object.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->setConfig('field', $form_state->getValue('field'));
    $this->setConfig('style', $form_state->getValue('style'));
    $this->setConfig('types', $form_state->getValue('types'));
    $this->setConfig('terms', $form_state->getValue('terms'));
  }

  /**
   * Configuration get wrapper.
   *
   * @param string $key
   *   The field name.
   *
   * @return mixed
   *   The value of the supplied field.
   */
  protected function getConfig($key = '') {
    $cfg = &drupal_static(__CLASS__ . '_' . __FUNCTION__, $this->factory->get('pager.cfg'));

    return $key ? $cfg->get($key) : (object) $cfg->get();
  }

  /**
   * Build a content type to label hash.
   *
   * @return array
   *   A content type to label hash.
   */
  protected function getContentTypes() {
    $types = [];

    foreach ($this->dbh->selectContentTypes() as $type) {
      if (preg_match('/^[a-z][a-z_]+[a-z]$/', $type)) {
        $types[$type] = ucwords($type);
      }
    }
    return $types;
  }

  /**
   * Build a image field name to label hash.
   *
   * @return array
   *   An image field name to label hash.
   */
  protected function getImageFields() {
    $fields = [];

    foreach ($this->dbh->selectImgData() as $data) {
      $obj = (object) unserialize($data);

      if ($obj->field_type == 'image' && $obj->entity_type == 'node') {
        $fields[$obj->field_name] = $obj->label;
      }
    }
    return $fields;
  }

  /**
   * Build a image style to label hash.
   *
   * @return array
   *   An image style to label hash.
   */
  protected function getImageStyles() {
    $styles = [];

    foreach (ImageStyle::loadMultiple() as $key => $style) {
      $styles[$key] = $style->get('label');
    }
    return $styles;
  }

  /**
   * Build a image term name to label hash.
   *
   * @return array
   *   An image field name to label hash.
   */
  protected function getTerms() {
    $terms = [];

    foreach ($this->dbh->selectTerms() as $obj) {
      $terms[$obj->tid] = ucwords(preg_replace('/[^a-z]+/', ' ', $obj->vid)) . ': ' . $obj->name;
    }
    return $terms;
  }

  /**
   * Configuration get wrapper.
   *
   * @param string $key
   *   The field name.
   * @param mixed $val
   *   The field value.
   *
   * @return mixed
   *   The value of the supplied field.
   */
  protected function setConfig($key = '', $val = NULL) {
    $cfg = &drupal_static(__CLASS__ . '_' . __FUNCTION__, $this->factory->getEditable('pager.cfg'));

    return $key ? $cfg->set($key, $val)->save() : NULL;
  }

}
