<?php

namespace Drupal\pager\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Routing\CurrentRouteMatch;
use Drupal\Core\Url;
use Drupal\image\Entity\ImageStyle;
use Drupal\pager\PagerStorage;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Provides a Pager block.
 *
 * @Block(
 *   id = "pager",
 *   admin_label = @Translation("Pager Block")
 * )
 */
class PagerBlock extends BlockBase implements ContainerFactoryPluginInterface {
  protected $dbh;
  protected $cfg;
  protected $entityMgr;
  protected $request;
  protected $route;

  /**
   * Extended constructor for the PageBlock class.
   *
   * @param array $config
   *   The block configuration.
   * @param string $plugin_id
   *   The plugin ID.
   * @param string $plugin_def
   *   The plugin definition.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request
   *   The http request.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $factory
   *   The pager settings.
   * @param \Drupal\Core\Routing\CurrentRouteMatch $route
   *   The current route service.
   * @param \Drupal\Core\Entity\EntityTypeManager $entityMgr
   *   The manager used to load a node.
   * @param \Drupal\pager\PagerStorage $dbh
   *   The storage interface.
   */
  public function __construct(array $config, $plugin_id, $plugin_def, RequestStack $request, ConfigFactoryInterface $factory, CurrentRouteMatch $route, EntityTypeManager $entityMgr, PagerStorage $dbh) {
    parent::__construct($config, $plugin_id, $plugin_def);
    $this->dbh = $dbh;
    $this->cfg = (object) $factory->get('pager.cfg')->get();
    $this->entityMgr = $entityMgr;
    $this->request = $request->getCurrentRequest();
    $this->route = $route;
  }

  /**
   * BlockBase extended constructor.
   *
   * @param Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The container interface.
   * @param array $config
   *   The block configuration.
   * @param string $plugin_id
   *   The plugin ID.
   * @param string $plugin_def
   *   The plugin definition.
   */
  public static function create(ContainerInterface $container, array $config, $plugin_id, $plugin_def) {
    return new static($config, $plugin_id, $plugin_def, $container->get('request_stack'), $container->get('config.factory'), $container->get('current_route_match'), $container->get('entity_type.manager'), $container->get('pager.storage'));
  }

  /**
   * Build the pager block render array.
   *
   * @return array
   *   A Drupal render array.
   */
  public function build() {
    $data = [];
    $node = $this->route->getParameter('node');

    $tid = $this->getTid($node);

    if (!$tid) {
      return [];
    }
    $prev = $this->dbh->selectPrev($node->id(), $tid, $this->cfg->types);

    if ($prev) {
      $data[] = $this->getNavItem($prev, 'pager-prev-node');
    }
    $next = $this->dbh->selectNext($node->id(), $tid, $this->cfg->types);

    if ($next) {
      $data[] = $this->getNavItem($next, 'pager-next-node');
    }
    if (!empty($data)) {
      return [
        '#theme'    => 'pager_block',
        '#data'     => $data,
        '#cache'    => ['max-age' => 0],
        '#attached' => ['library' => ['pager/drupal.pager-links']],
      ];
    }
    return [];
  }

  /**
   * Process the node title for the image alt attribute.
   *
   * @param string $title
   *   The node title.
   *
   * @return string
   *   A filterd title.
   */
  protected function filterAlt($title) {
    $regx = [
      '/[\'\"]+/'    => '',
      '/[^\w\-\.]+/' => ' ',
      '/[_\s]+/'     => ' ',
      '/^\s+|\s+$/'  => '',
    ];
    return preg_replace(array_keys($regx), array_values($regx), $title);
  }

  /**
   * Select the previous and next node IDs.
   *
   * @param object $node
   *   The node object.
   *
   * @return string
   *   The image URL.
   */
  protected function getImgUrl($node) {
    $img = $node ? $node->get($this->cfg->field)->getValue() : [['target_id' => 0]];
    $file = !empty($img[0]['target_id']) ? $this->entityMgr->getStorage('file')->load($img[0]['target_id']) : NULL;

    return $file ? ImageStyle::load($this->cfg->style)->buildUrl($file->getFileUri()) : '';
  }

  /**
   * Build a nav item.
   *
   * @param int $nid
   *   The node ID.
   * @param string $class
   *   The list item class.
   *
   * @return object
   *   A nav item object.
   */
  protected function getNavItem($nid, $class = '') {
    $node = $this->entityMgr->getStorage('node')->load($nid);

    return (object) [
      'alt'   => $node ? $this->filterAlt($node->title->value) : '',
      'class' => $class,
      'href'  => $node ? $this->getNodeUrl($node->id()) : '',
      'src'   => $node ? $this->getImgUrl($node) : '',
      'title' => $node ? $node->title->value : '',
    ];
  }

  /**
   * Select the previous and next node IDs.
   *
   * @param int $nid
   *   The node ID.
   *
   * @return string
   *   The node URL.
   */
  protected function getNodeUrl($nid) {
    return Url::fromRoute('entity.node.canonical', ['node' => $nid], ['absolute' => TRUE])->toString();
  }

  /**
   * Get the term ID.
   *
   * @param object $node
   *   The node object.
   *
   * @return string
   *   The image URL.
   */
  protected function getTid($node) {
    if (!in_array($node->getType(), $this->cfg->types)) {
      return 0;
    }
    return $this->dbh->selectTid($node->id(), $this->cfg->terms);
  }

}
